/* --- [ usi_logger.c ] -------------------------------------------------------
* Datum       		  : 09.06.2008
*							 Fehler beseitigt, der 0x3e im Datenstrom als TWI-STOP interpretiert
*                     11.06.2008
* 							 gesamte Programmlogik vereinfacht, Konzept der Datenpufferung gendert
*
* AppNote           : AVR312 - Using the USI module as a I2C slave
*                     benutzt die usi-slave.c von Frank Jonischkies
* Compiler			  : avr-gcc 4.1.2
*		
* Description       : lauscht passiv am TWI-Bus, zeichnet den Datenverkehr mit
*							 und protokolliert auf COM.
*							 Im 2 Ringpuffern von je 50 Byte knnen 50 Datenbyte aufgezeichnet werden 
*							 
* Tabwidth          : 3
*
****************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <string.h>
#include <avr/pgmspace.h>
#include "usi_logger_slave.h"

#define STARTMSG  "USI-Logger @ 38400"
#define BAUD  38400
#define TEILER ((F_CPU /(BAUD * 16l)) -1) 
#define CALC_UBBR(F_CPU, BAUD) ((F_CPU /(BAUD * 16)) -1)

uint8_t TWI_control[BUFFER_SIZE];			// Array fr die Steuerzeichen START, ACK, NACK, STOP
uint8_t TWI_data[BUFFER_SIZE];				// Array fr die Daten
volatile uint8_t RxTail;						// der gemeinsame Index 			
volatile uint8_t RxHead;														

char msg[4];										// nur fr die Zahlenumwandlung -> dreistellig + /0

// ----------------------------------------------------------------------------
// ein einzelnes Zeichen auf COM ausgeben
void putc(uint8_t Zeichen)
{
while(!(UCSRA &(1<<UDRE)));
UDR = Zeichen;
}

// ----------------------------------------------------------------------------
// einen String auf COM ausgeben
void putstring(char *s)
{
while(*s)
	{
	putc(*s);
	s++;
	}
}

// ----------------------------------------------------------------------------
// gibt einen String aus dem Flash aus
void puts_p(const char *progmem_s)
{
register char c;
while ( (c = pgm_read_byte(progmem_s++)) )
	{
   putc(c);
	}
}

// ----------------------------------------------------------------------------
int main( void )
{
register uint8_t index;								// variable fr volatile RxTail = Index auf das Array
register uint8_t control;							// Variable fr TWI_control[]
uint8_t counter = 0;									// Zeilenzhler fr die Ausgabe
															// alle anderen Variablen sind global deklariert
usi_twi_slave_init();
sei();	 												// enable_interrupt

UBRRL  = TEILER;										// serielle Schnittstelle initialisieren
UCSRB |= (1<<TXEN);
UCSRC |= (0<<UMSEL) | (1<<UCSZ1) | (1<<UCSZ0);
															// und eine Meldung ausgeben
putc(13);												// Zeilenumbruch									
puts_p(PSTR(STARTMSG));								// Startmeldung als Lebenszeichen
putc(13);												// Zeilenumbruch	

while(1)
	{
	if (!(RxTail == RxHead))						// testen, ob ein Byte empfangen
		{													// wenn ja, dann:
		
		index = RxTail;								// volatile RxTail in Register laden 
		if ((++index) >= BUFFER_SIZE)index = 0;// Index incrementieren und Grenzen prfen
		RxTail = index; 								// genderten Wert zurck ins SRAM
		
		control = TWI_control[index];				// volatile TWI_control in Register laden
															// (das macht der Compiler vermutlich schon von sich aus ?)
		if (control == START)						// bei START Zeilenumbruch, Zeilennummer, Startsymbol ausgeben
			{
			putc(13);									// Zeilenumbruch
			if (counter <  10) putc('0');			// Zeilennummer immer 2-stellig ausgeben
 			putstring(itoa(counter, msg, 10));	// Zeilenzhler
			putc(' ');									// Leerzeichen
			putc(CHR_START);							// START-Symbol
			if (TWI_data[index] < 10) putc('0');// eine fhrende 0
			putstring(itoa(TWI_data[index], msg, 16)); // die Master-Adresse									
			if (++counter > 99) counter = 0;		// den counter incrementieren, Maximalwert 99 !
			}

		else if (control == ACK)					// ACK und DATA ausgeben
			{
			putc(CHR_ACK);								// ACK
			if (TWI_data[index] < 10) putc('0');// fhrende 0
			putstring(itoa(TWI_data[index], msg, 16));// Datenbyte
			}
			
		else if (control == NACK)					// NACK und DATA ausgeben
			{
			putc(CHR_NACK);
			if (TWI_data[index] < 10) putc('0');
			putstring(itoa(TWI_data[index], msg, 16));
			}			
		
		else if (control == STOP)					// in TWI_data steht das vorausgegangen ACK bzw. NACK
			{
			if (TWI_data[index] == ACK) putc(CHR_ACK); else putc(CHR_NACK);
			putc(CHR_STOP);							// STOP ausgeben
			}
			
		else
			{
			putc('?');									// dann liegt ein Fehler vor
			}
		}
	}
}	


